package com.redhat.thermostat.tools.eclipse.plugin.wizards;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.concurrent.TimeUnit;
import java.util.zip.ZipFile;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;

import com.redhat.thermostat.tools.eclipse.plugin.Activator;
import com.redhat.thermostat.tools.eclipse.plugin.Unzipper;

public class ExportPluginWizard extends Wizard implements IExportWizard {

    private IStructuredSelection selection;
    private ExportPluginPage exportPage;

    @Override
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        setWindowTitle("Export Thermostat Plugin");
        setNeedsProgressMonitor(true);

        this.selection = selection;
    }

    @Override
    public void addPages() {
        exportPage = new ExportPluginPage(selection);
        addPage(exportPage);
    }

    @Override
    public boolean performFinish() {
        IProject project = exportPage.getProject();
        String targetLocation = exportPage.getInstallationLocation();

        Job export = new ExportPlugin(project, targetLocation);
        export.schedule();

        return true;
    }

    private static class ExportPlugin extends Job {

        private IProject project;
        private String targetLocation;

        public ExportPlugin(IProject project, String targetLocation) {
            super("Exporting Thermostat plugin");
            this.project = project;
            this.targetLocation = targetLocation;
        }

        @Override
        protected IStatus run(IProgressMonitor monitor) {
            monitor.beginTask("Exporting plugin", IProgressMonitor.UNKNOWN);

            MessageConsole console = getConsole("Exporting " + project.getName());
            MessageConsoleStream stream = console.newMessageStream();

            try {
                // mvn clean package
                boolean mavenWorked = new MavenRunner().run(stream, project, "package");
                if (!mavenWorked) {
                    stream.write("Faild to run maven");
                    return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Unable to run 'mvn clean package'");
                }
            } catch (CoreException | IOException e) {
                e.printStackTrace();
                return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Unable to run 'mvn clean package'", e);
            }

            if (monitor.isCanceled()) {
                return Status.CANCEL_STATUS;
            }

            monitor.worked(1);

            // FIXME don't hardcode distribution project name
            final String distributionModuleName = project.getName() + "-distribution";
            File projectDir = project.getLocation().toFile();
            File distributionDir = new File(projectDir, distributionModuleName);
            File distributionTargetDir = new File(distributionDir, "target");

            // FIXME find the zip matching the exact pom version

            String[] files = distributionTargetDir.list(new FilenameFilter() {
                @Override
                public boolean accept(File dir, String name) {
                    return name.matches("\\Q" + distributionModuleName + "\\E.*\\.zip");
                }
            });

            String zipFile = files[0];

            if (monitor.isCanceled()) {
                return Status.CANCEL_STATUS;
            }

            monitor.worked(1);

            try {
                ZipFile pluginZip = new ZipFile(new File(distributionTargetDir, zipFile));
                File installationDirectory = new File(targetLocation, "plugins");
                new Unzipper().unzip(pluginZip, installationDirectory);
                stream.write("Done installing " + project.getName() + " to " + installationDirectory + "\n");
            } catch (IOException e) {
                e.printStackTrace();
                return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Unable to install plugin in target location", e);
            }

            monitor.worked(1);

            return Status.OK_STATUS;
        }

        private MessageConsole getConsole(String name) {
            IConsoleManager consoleManager = ConsolePlugin.getDefault().getConsoleManager();
            MessageConsole console = getOrCreateConsole(consoleManager, name);
            consoleManager.showConsoleView(console);
            return console;
        }

        private MessageConsole getOrCreateConsole(IConsoleManager consoleManager, String name) {
            IConsole[] consoles = consoleManager.getConsoles();
            for (IConsole console : consoles) {
                if (console.getName().equals(name)) {
                    return (MessageConsole) console;
                }
            }

            MessageConsole console = new MessageConsole(name, null);
            consoleManager.addConsoles(new IConsole[] { console });
            return console;
        }

    }

    private static class MavenRunner {

        /** Returns true if completed successfully */
        public boolean run(MessageConsoleStream stream, IProject project, String goal) throws CoreException, IOException {
            try {
                stream.write("Building " + project.getName() + "\n");
                ConsoleRedirector consoleRedirector = new ConsoleRedirector(stream);

                String[] command = new String[] { "mvn", "clean", goal };

                Process p = DebugPlugin.exec(command, project.getLocation().toFile());
                Launch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
                IProcess launchProcess = DebugPlugin.newProcess(launch, p, "Export Thermostat Plugin");

                launchProcess.getStreamsProxy().getOutputStreamMonitor().addListener(consoleRedirector);
                launchProcess.getStreamsProxy().getErrorStreamMonitor().addListener(consoleRedirector);

                int exitValue = p.waitFor();
                return (p.exitValue() == 0);

            } catch (InterruptedException e) {
                IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Error running maven", e);
                throw new CoreException(status);
            }
        }

    }

    private static class ConsoleRedirector implements IStreamListener {

        private MessageConsoleStream stream;

        public ConsoleRedirector(MessageConsoleStream stream) {
            this.stream = stream;
        }

        @Override
        public void streamAppended(String text, IStreamMonitor monitor) {
            try {
                stream.write(text);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }
}
